package net.lensfrex.oj.collector;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import net.lensfrex.oj.collector.data.Result;
import net.lensfrex.oj.collector.utils.NetworkUtil;
import org.apache.http.client.utils.URIBuilder;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Collector {

    private static final String API_BASE = "https://oj.wust-acm.top/api/problem";

    private static final int PAGE_LIMIT = 100;

    private String generatePageRequestUrl(String baseUrl, int offset, int limit, int page) throws URISyntaxException {
        URIBuilder uriBuilder = new URIBuilder(baseUrl);
        uriBuilder.addParameter("paging", "true");
        uriBuilder.addParameter("offset", String.valueOf(offset));
        uriBuilder.addParameter("limit", String.valueOf(limit));
        uriBuilder.addParameter("page", String.valueOf(page));

        return uriBuilder.toString();
    }

    private HashMap<String, String> getHeaders() {
        HashMap<String, String> headers = new HashMap<>();

        headers.put("User-Agent", "lensfrex/0.0.1 CanYouSeeMe/0.0.2 IJustLearning/0.0.3");
        headers.put("Host", "oj.wust-acm.top");
        headers.put("Accept-Encoding", "gzip, deflate, br");

        return headers;
    }

    private Result[] getResults(String url, Map<String, String> headers) throws IOException {
        String json = NetworkUtil.get(url, headers);

        JsonArray resultJsonArray = JsonParser.parseString(json).getAsJsonObject()
                .getAsJsonObject("data")
                .getAsJsonArray("results");

        return new Gson().fromJson(resultJsonArray, Result[].class);
    }

    public ArrayList<Result> collectAllResults() {
        ArrayList<Result> results = new ArrayList<>();
        try {
            String url = generatePageRequestUrl(API_BASE, 0, 1, 1);
            HashMap<String, String> headers = getHeaders();

            String json = NetworkUtil.get(url, headers);

            int totalResult = JsonParser.parseString(json).getAsJsonObject()
                    .getAsJsonObject("data")
                    .getAsJsonPrimitive("total").getAsInt();

            int page = (totalResult / PAGE_LIMIT) + (totalResult % PAGE_LIMIT == 0 ? 0 : 1);


            System.out.println("Total results: " + totalResult);
            System.out.println("Page Limit: " + PAGE_LIMIT);
            System.out.println("Total pages: " + page);

            for (int i = 1; i <= page; i++) {
                int offset = (i - 1) * PAGE_LIMIT;
                System.out.println("\n------------------------------------------------------");
                System.out.println("Getting page " + i);
                System.out.println("Offset " + offset);

                url = generatePageRequestUrl(API_BASE, offset, PAGE_LIMIT, i);
                System.out.println("Request URL: " + url);

                Result[] results1 = getResults(url, headers);

                results.addAll(Arrays.asList(results1));

                System.out.println("Successful get " + results1.length + " result(s)");
            }
        } catch (URISyntaxException | IOException e) {
            throw new RuntimeException(e);
        }

        return results;
    }
}
